library(readr)

args <- commandArgs(trailingOnly = TRUE)
dir <- args[1]
script.dir <- dir
setwd(dir)

process_project <- function(name) {
  filename <- paste(script.dir, "/rawData/stats_",name,".csv", sep="")
  
  project_data <- new.env()
  
  allData <- read_delim(filename, 
                        " ", escape_double = FALSE, na = "N/A", 
                        trim_ws = TRUE)
  
  allData<-allData[!is.na(allData$author_changes),]
  
  #NEW is defined here
  allData$isNew <- (allData$author_changes <= 2)
  
  #Main function to compare the distributions
  do_compare_distributions <- function(data, columnName, splitBy) {
    col_names <- c(splitBy, columnName)
    distrAndFlag <- data[col_names]
    
    parameterKey <- paste(columnName, "_for_", splitBy, sep='')
    pValueKey <- paste(parameterKey, "_pVal", sep='')
    
    newDistr <- subset(distrAndFlag, distrAndFlag[[splitBy]] == TRUE, select = columnName)[[columnName]]
    oldDistr <- subset(distrAndFlag, distrAndFlag[[splitBy]] == FALSE, select = columnName)[[columnName]]
    
    twotailed <- wilcox.test(newDistr, oldDistr)
    pvalue <- twotailed$p.value
    #double threshold for two-tailed
    threshold_pvalue <- 0.01
    
    cat(paste("P-value for two-tailed test: ", pvalue, "\n"))
    if (pvalue < threshold_pvalue*2) {
      cat("Two-tailed test shows significant difference. Trying one-tailed tests now to determine direction of difference\n")
      lessTest <- wilcox.test(newDistr, oldDistr, alternative = "less")
      lessP <- lessTest$p.value
      if (lessP < threshold_pvalue) {
        cat(paste(columnName, " for new developers is LESS. P-value: ", lessP, "\n"))
        return
        project_data[[parameterKey]] <- "LESS"
        project_data[[pValueKey]] <- lessP
      }
      moreTest <- wilcox.test(newDistr, oldDistr, alternative = "greater")
      moreP <- moreTest$p.value
      if (moreP < threshold_pvalue) {
        cat(paste(columnName, " for new developers is GREATER. P-value: ", moreP, "\n"))
        return
        project_data[[parameterKey]] <- "GREATER"
        project_data[[pValueKey]] <- moreP
      }
    } else {
      project_data[[parameterKey]] <- "same"
      project_data[[pValueKey]] <- (pvalue)
      cat("No significant difference.\n")
    }
    
    
    
    cat("-------------------------\n")
  }
  
  compare_distributions <- function(data, columnName){
    cat("NEW (<= 2 changes): \n")
    do_compare_distributions(data, columnName, "isNew")
  }
  
  #ACTUAL ANALYSIS
  #summary(allData$isNew)
  cat("=======\nRQ1:\n=======\n")
  cat("Fraction of all review events made by reviewer of current change:\n")
  compare_distributions(allData, "reviewer_fractionOfMeanReviews")
  
  cat("Fraction of all changes made by reviewer of current change:\n")
  compare_distributions(allData, "reviewer_fractionOfMeanChanges")
  
  cat("=======\nRQ2:\n=======\n")
  cat("Removing duplications in review IDs to only count each review once.\n\n")
  
  allData$uniqueKey <- paste("review_", allData$review_id, "_author_", allData$author_id, sep = "")
  
  dataUnique <- allData[!duplicated(allData$uniqueKey),]
  cat("Review comments count:\n")
  compare_distributions(dataUnique, "review_commentsCount")
  
  cat("Reviewers count:\n")
  compare_distributions(dataUnique, "review_reviewersCount")
  
  cat("Time to first comment:\n")
  compare_distributions(dataUnique, "review_timeToFirstComment")
  
  cat("Review lifespan time:\n")
  compare_distributions(dataUnique, "review_lifespanT")
  
  
  cat("=======\nRQ3:\n=======\n")
  mergeRate_for_new <- mean(dataUnique$review_isMerged[dataUnique$isNew == TRUE])
  mergeRate_for_not_new <- mean(dataUnique$review_isMerged[dataUnique$isNew == FALSE])
  project_data[["mergeRate_new"]] <- mergeRate_for_new
  project_data[["mergeRate_not_new"]] <- mergeRate_for_not_new
  cat("review merge rate for NEW: \n")
  print(mergeRate_for_new)
  cat("review merge rate for NOT NEW: \n")
  print(mergeRate_for_not_new)
  
  
  totalChanges <- nrow(dataUnique)
  changesByNewcomers <- dataUnique[dataUnique$isNew == TRUE,]
  
  totalMerged <- nrow(dataUnique[dataUnique$review_isMerged == TRUE,])
  byNewcomersMerged <- nrow(changesByNewcomers[changesByNewcomers$review_isMerged == TRUE,])
  
  totalAuthors <- nrow(allData[!duplicated(allData$author_id),])
  totalReviewers <- nrow(dataUnique[!duplicated(dataUnique$reviewer_id),])
  print("Unique authors:")
  print(totalAuthors)
  print("Unique reviewers:")
  print(totalReviewers)
  
  dataNew <- dataUnique[dataUnique$isNew == TRUE,]
  
  newcomerAuthors <- nrow(dataNew[!duplicated(dataNew$author_id),])
  reviewersForNewcomerChanges <- nrow(dataNew[!duplicated(dataNew$reviewer_id),])
  
  print("Newcomer authors:")
  print(newcomerAuthors)
  print("Reviewers for newcomer changes:")
  print(reviewersForNewcomerChanges)
  
  project_data[["authorsCount"]] <- totalAuthors
  project_data[["reviewersCount"]] <- totalReviewers
  project_data[["newcomerAuthorsCount"]] <- newcomerAuthors
  project_data[["reviewersForNewcomersCount"]] <- reviewersForNewcomerChanges
  
  project_data[["totalChanges"]] <- totalChanges
  project_data[["changesByNewcomers"]] <- nrow(changesByNewcomers)
  project_data[["totalMerged"]] <- totalMerged
  project_data[["byNewcomersMerged"]] <- byNewcomersMerged
  
  cat("NEW:\n")
  print(summary(allData$isNew))
  
  print(summary(allData$median_changes))
  print(summary(allData$mean_changes))
  
  print(as.vector(ls(project_data)))
  
  return(project_data)
}

projects_data = new.env()
headers <- c()

allFiles <- list.files(path = paste(script.dir,"/rawData",sep=""), pattern="info_.*")
names = gsub("info_", "", allFiles)
names = gsub(".txt", "", names)

for (name in names) {
  cat(paste("\n\n\n\nProject: ", name, "\n"))
  projectData <- process_project(name)
  
  prjInfoVec <- scan(paste(script.dir,"/rawData/","info_", name, ".txt", sep=''), what = "character")
  
  changesCount <- as.integer(prjInfoVec[2])
  reviewsCount <- as.integer(prjInfoVec[3])
  firstChangeDate <- as.double(prjInfoVec[4])
  firstReviewDate <- as.double(prjInfoVec[5])
  
  projectData[["_changesCount"]] <- changesCount
  projectData[["_reviewsCount"]] <- reviewsCount
  projectData[["_firstChangeDate"]] <- firstChangeDate
  projectData[["_firstReviewDate"]] <- firstReviewDate
  
  projects_data[[name]] <- projectData
  headers <- ls(projectData)
}

outFile <- paste(script.dir, "/out.txt", sep='')
con <- file(outFile, "w")

cat("name ", file=con)
for (header in headers) {
  cat(header, file=con)
  cat(' ', file=con)
}
cat("\n", file=con)

for (name in names) {
  cat(name, file=con)
  cat(' ', file=con)
  for (header in headers) {
    cat(projects_data[[name]][[header]], file=con)
    cat(' ', file=con)
  }
  cat('\n', file=con)
}

close(con)


